==> scala-k8s-objects_sjs1_2.13-0.1.0-sources.jar: io/k8s/api/rbac/v1/ClusterRole.scala <==
package io.k8s.api.rbac.v1

import dev.hnaderi.k8s._

/** ClusterRole is a cluster level, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding or ClusterRoleBinding. */
final case class ClusterRole(
  aggregationRule : Option[io.k8s.api.rbac.v1.AggregationRule] = None,
  metadata : Option[io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta] = None,
  rules : Option[Seq[io.k8s.api.rbac.v1.PolicyRule]] = None
) extends KObject {
  protected val _resourceKind = ResourceKind("rbac.authorization.k8s.io", "ClusterRole", "v1")


  /** Returns a new data with aggregationRule set to new value */
  def withAggregationRule(value: io.k8s.api.rbac.v1.AggregationRule) : ClusterRole = copy(aggregationRule = Some(value))
  /** if aggregationRule has a value, transforms to the result of function*/
  def mapAggregationRule(f: io.k8s.api.rbac.v1.AggregationRule => io.k8s.api.rbac.v1.AggregationRule) : ClusterRole = copy(aggregationRule = aggregationRule.map(f))

  /** Returns a new data with metadata set to new value */
  def withMetadata(value: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : ClusterRole = copy(metadata = Some(value))
  /** if metadata has a value, transforms to the result of function*/
  def mapMetadata(f: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta => io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : ClusterRole = copy(metadata = metadata.map(f))

  /** Returns a new data with rules set to new value */
  def withRules(value: Seq[io.k8s.api.rbac.v1.PolicyRule]) : ClusterRole = copy(rules = Some(value))
  /** Appends new values to rules */
  def addRules(newValues: io.k8s.api.rbac.v1.PolicyRule*) : ClusterRole = copy(rules = Some(rules.fold(newValues)(_ ++ newValues)))
  /** if rules has a value, transforms to the result of function*/
  def mapRules(f: Seq[io.k8s.api.rbac.v1.PolicyRule] => Seq[io.k8s.api.rbac.v1.PolicyRule]) : ClusterRole = copy(rules = rules.map(f))
}

==> scala-k8s-objects_sjs1_2.13-0.4.0-sources.jar: io/k8s/api/rbac/v1/ClusterRole.scala <==
package io.k8s.api.rbac.v1

import dev.hnaderi.k8s._
import dev.hnaderi.k8s.utils._

/** ClusterRole is a cluster level, logical grouping of PolicyRules that can be referenced as a unit by a RoleBinding or ClusterRoleBinding. */
final case class ClusterRole(
  aggregationRule : Option[io.k8s.api.rbac.v1.AggregationRule] = None,
  metadata : Option[io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta] = None,
  rules : Option[Seq[io.k8s.api.rbac.v1.PolicyRule]] = None
) extends KObject {
  protected val _resourceKind = ResourceKind("rbac.authorization.k8s.io", "ClusterRole", "v1")


  /** Returns a new data with aggregationRule set to new value */
  def withAggregationRule(value: io.k8s.api.rbac.v1.AggregationRule) : ClusterRole = copy(aggregationRule = Some(value))
  /** if aggregationRule has a value, transforms to the result of function*/
  def mapAggregationRule(f: io.k8s.api.rbac.v1.AggregationRule => io.k8s.api.rbac.v1.AggregationRule) : ClusterRole = copy(aggregationRule = aggregationRule.map(f))

  /** Returns a new data with metadata set to new value */
  def withMetadata(value: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : ClusterRole = copy(metadata = Some(value))
  /** if metadata has a value, transforms to the result of function*/
  def mapMetadata(f: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta => io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : ClusterRole = copy(metadata = metadata.map(f))

  /** Returns a new data with rules set to new value */
  def withRules(value: Seq[io.k8s.api.rbac.v1.PolicyRule]) : ClusterRole = copy(rules = Some(value))
  /** Appends new values to rules */
  def addRules(newValues: io.k8s.api.rbac.v1.PolicyRule*) : ClusterRole = copy(rules = Some(rules.fold(newValues)(_ ++ newValues)))
  /** if rules has a value, transforms to the result of function*/
  def mapRules(f: Seq[io.k8s.api.rbac.v1.PolicyRule] => Seq[io.k8s.api.rbac.v1.PolicyRule]) : ClusterRole = copy(rules = rules.map(f))

  override def foldTo[T : Builder] : T = ClusterRole.encoder.apply(this)
}

object ClusterRole {

    implicit val encoder : Encoder[io.k8s.api.rbac.v1.ClusterRole] = new Encoder[io.k8s.api.rbac.v1.ClusterRole] {
        def apply[T : Builder](o: io.k8s.api.rbac.v1.ClusterRole) : T = {
          val obj = ObjectWriter[T]()
          obj
            .write("aggregationRule", o.aggregationRule)
            .write("metadata", o.metadata)
            .write("rules", o.rules)
            .write("kind", o.kind)
            .write("apiVersion", o.apiVersion)
            .build
        }
    }

    implicit val decoder: Decoder[ClusterRole] = new Decoder[ClusterRole] {
      def apply[T : Reader](t: T): Either[String, ClusterRole] = for {
          obj <- ObjectReader(t)
          aggregationRule <- obj.readOpt[io.k8s.api.rbac.v1.AggregationRule]("aggregationRule")
          metadata <- obj.readOpt[io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta]("metadata")
          rules <- obj.readOpt[Seq[io.k8s.api.rbac.v1.PolicyRule]]("rules")
      } yield ClusterRole (
          aggregationRule = aggregationRule,
          metadata = metadata,
          rules = rules
        )
    }
}
